@@ -4,20 +4,22 @@ from scipy.sparse.linalg import spsolve, LinearOperator, gmres
 from scipy.linalg import norm
 from scipy.sparse import diags
 import matplotlib.pyplot as plt
+import types
 
 
 # generic class only implement algo. didn't implement detail func about M0, UT, F1 or F2
 class MfgBase(object):
     # it's a base class which implements the policy iteration and fixed point iteration
-    def __init__(self, Q_guess=None, nv=0.05, Nx=200, Nt=200, T=1.0, bc="periodic"):
-        self.Nx = Nx
+    def __init__(
+        self, Q_guess=None, nv=0.05, x1=0, x2=1, Nx=200, Nt=200, T=1.0, bc="periodic"
+    ):
         self.Nt = Nt
         if bc == "periodic":
-            self.xs = np.linspace(0, 1, Nx + 1)[:-1]  # grid on space
+            self.xs = np.linspace(x1, x2, Nx + 1)[:-1]  # grid on space
         elif bc == "dirichlet":
-            self.xs = np.linspace(0, 1, Nx + 1)[1:-1]  # grid on space
+            self.xs = np.linspace(x1, x2, Nx + 1)[1:-1]  # grid on space
         elif bc == "neumann":
-            self.xs = np.linspace(0, 1, Nx + 1)  # grid on space
+            self.xs = np.linspace(x1, x2, Nx + 1)  # grid on space
         self.Nx = self.xs.shape[0]
         self.dx = self.xs[1] - self.xs[0]
         self.ts = np.linspace(0, T, Nt + 1)  # grid on time
@@ -43,7 +45,12 @@ class MfgBase(object):
         self.U_old = np.copy(self.U)
 
         # initial guess about Q
-        if Q_guess:
+        if isinstance(Q_guess, types.FunctionType):
+            q0 = Q_guess(self.xs)
+            q0 = np.tile(q0, (self.Nt, 1))
+            self.QL = np.maximum(q0, 0)
+            self.QR = np.minimum(q0, 0)
+        elif isinstance(Q_guess, np.ndarray):
             self.QL = np.copy(Q_guess[0])
             self.QR = np.copy(Q_guess[1])
         else:
@@ -477,22 +484,24 @@ class MfgBase2D(object):
     def __init__(
         self,
         Q_guess=None,
-        nv=0.05,
-        Nx=200,
-        Nt=200,
+        nv=0.3,
+        x1=0,
+        x2=1,
+        Nx=50,
+        Nt=50,
         T=1.0,
         bc="periodic",
     ):
         self.Nt = Nt
         if bc == "periodic":
-            self.xs1 = np.linspace(0, 1, Nx + 1)[:-1]  # grid on space
-            self.xs2 = np.linspace(0, 1, Nx + 1)[:-1]  # grid on space
+            self.xs1 = np.linspace(x1, x2, Nx + 1)[:-1]  # grid on space
+            self.xs2 = np.linspace(x1, x2, Nx + 1)[:-1]  # grid on space
         elif bc == "dirichlet":
-            self.xs1 = np.linspace(0, 1, Nx + 1)[1:-1]  # grid on space
-            self.xs2 = np.linspace(0, 1, Nx + 1)[1:-1]  # grid on space
+            self.xs1 = np.linspace(x1, x2, Nx + 1)[1:-1]  # grid on space
+            self.xs2 = np.linspace(x1, x2, Nx + 1)[1:-1]  # grid on space
         elif bc == "neumann":
-            self.xs1 = np.linspace(0, 1, Nx + 1)  # grid on space
-            self.xs2 = np.linspace(0, 1, Nx + 1)  # grid on space
+            self.xs1 = np.linspace(x1, x2, Nx + 1)  # grid on space
+            self.xs2 = np.linspace(x1, x2, Nx + 1)  # grid on space
         self.Nx = self.xs1.shape[0]
         self.dx = self.xs1[1] - self.xs1[0]
         self.ts = np.linspace(0, T, Nt + 1)  # grid on time
@@ -518,7 +527,18 @@ class MfgBase2D(object):
         self.U_old = np.copy(self.U)
 
         # initial guess about Q
-        if Q_guess:
+        if isinstance(Q_guess, types.FunctionType):
+            xs1, xs2 = np.meshgrid(self.xs1, self.xs2)
+            q0_1, q0_2 = Q_guess(xs1, xs2)
+            q0_1 = q0_1.reshape(-1)
+            q0_2 = q0_2.reshape(-1)
+            q0_1 = np.tile(q0_1, (self.Nt, 1))
+            q0_2 = np.tile(q0_2, (self.Nt, 1))
+            self.QL1 = np.maximum(q0_1, 0)
+            self.QR1 = np.minimum(q0_1, 0)
+            self.QL2 = np.maximum(q0_2, 0)
+            self.QR2 = np.minimum(q0_2, 0)
+        elif isinstance(Q_guess, np.ndarray):
             self.QL1 = np.copy(Q_guess[0])
             self.QR1 = np.copy(Q_guess[1])
             self.QL2 = np.copy(Q_guess[2])
